#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualnetwork_info

version_added: "0.0.1"

short_description: Get virtual network facts

description:
    - Get facts for a specific virtual network or all virtual networks within a resource group.

options:
    name:
        description:
            - Only show results for a specific virtual network.
        type: str
    resource_group:
        description:
            - Limit results by resource group. Required when filtering by name.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
    list_usage:
        description:
            - Whether lists usage stats. I(list_usage=true) to lists usage stats.
        type: bool
        default: False

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)

'''

EXAMPLES = '''
- name: Get facts for one virtual network
  azure_rm_virtualnetwork_info:
    resource_group: myResourceGroup
    name: secgroup001

- name: Get facts for all virtual networks
  azure_rm_virtualnetwork_info:
    resource_group: myResourceGroup

- name: Get facts by tags
  azure_rm_virtualnetwork_info:
    tags:
      - testing
'''
RETURN = '''
azure_virtualnetworks:
    description:
        - List of virtual network dicts.
    returned: always
    type: list
    example: [{
        "etag": 'W/"532ba1be-ae71-40f2-9232-3b1d9cf5e37e"',
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroup/myResourceGroup/providers/Microsoft.Network/virtualNetworks/vnet2001",
        "location": "eastus2",
        "name": "vnet2001",
        "properties": {
            "addressSpace": {
                "addressPrefixes": [
                    "10.10.0.0/16"
                ]
            },
            "provisioningState": "Succeeded",
            "resourceGuid": "a7ba285f-f7e7-4e17-992a-de4d39f28612",
            "subnets": []
        },
        "type": "Microsoft.Network/virtualNetworks"
    }]
virtualnetworks:
    description:
        - List of virtual network dicts with same format as M(azure.azcollection.azure_rm_virtualnetwork) module parameters.
    returned: always
    type: complex
    contains:
            id:
                description:
                    - Resource ID of the virtual network.
                sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/vnet2001
                returned: always
                type: str
            address_prefixes:
                description:
                    - List of IPv4 address ranges where each is formatted using CIDR notation.
                sample: ["10.10.0.0/16"]
                returned: always
                type: list
            dns_servers:
                description:
                    - Custom list of DNS servers.
                returned: always
                type: list
                sample: ["www.azure.com"]
            location:
                description:
                    - Valid Azure location.
                returned: always
                type: str
                sample: eastus
            tags:
                description:
                    - Tags assigned to the resource. Dictionary of string:string pairs.
                returned: always
                type: dict
                sample: { "tag1": "abc" }
            provisioning_state:
                description:
                    - Provisioning state of the resource.
                returned: always
                sample: Succeeded
                type: str
            flow_timeout_in_minutes:
                description:
                    - The FlowTimeout value (in minutes) for the Virtual Network.
                type: int
                returned: always
                sample: 8
            name:
                description:
                    - Name of the virtual network.
                returned: always
                type: str
                sample: foo
            subnets:
                description:
                    - Subnets associated with the virtual network.
                returned: always
                type: list
                contains:
                    id:
                        description:
                            - Resource ID of the subnet.
                        returned: always
                        type: str
                        sample: "/subscriptions/f64d4ee8-be94-457d-ba26-3fa6b6506cef/resourceGroups/v-xisuRG/providers/
                                Microsoft.Network/virtualNetworks/vnetb57dc95232/subnets/vnetb57dc95232"
                    name:
                        description:
                            - Name of the subnet.
                        returned: always
                        type: str
                        sample: vnetb57dc95232
                    provisioning_state:
                        description:
                            - Provisioning state of the subnet.
                        returned: always
                        type: str
                        sample: Succeeded
                    address_prefix:
                        description:
                            - The address prefix for the subnet.
                        returned: always
                        type: str
                        sample: '10.1.0.0/16'
                    address_prefixes:
                        description:
                            - Both IPv4 and IPv6 address prefixes for the subnet, will return null if only an  IPv4 set.
                        returned: always
                        type: list
                        sample: ["10.1.0.0/16", "fdda:e69b:2547:485e::/64"]
                    network_security_group:
                        description:
                            - Existing security group ID with which to associate the subnet.
                        returned: always
                        type: str
                        sample: null
                    route_table:
                        description:
                            - The reference of the RouteTable resource.
                        returned: always
                        type: str
                        sample: null
                    service_endpoints:
                        description:
                            - An array of service endpoints.
                        returned: always
                        type: list
                        sample: [
                        {
                            "locations": [
                                "southeastasia",
                                "eastasia"
                            ],
                            "service": "Microsoft.Storage"
                        }
                    ]
                    usages:
                        description:
                            - Lists usage stats.
                        type: list
                        returned: always
                        sample: [
                        {
                            "current_value": 0.0,
                            "id": "/subscriptions/xxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworks/vnet/subnets/default",
                            "limit": 251.0,
                            "name": {
                                "localized_value": "Subnet size and usage",
                                "value": "SubnetSpace"
                            }
                        }
                    ]
'''

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError
    from azure.mgmt.core.tools import parse_resource_id
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'VirtualNetwork'


class AzureRMNetworkInterfaceInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str'),
            list_usage=dict(type='bool', default=False),
        )

        self.results = dict(
            changed=False,
            virtualnetworks=[]
        )

        self.required_by = {'name': 'resource_group'}

        self.name = None
        self.resource_group = None
        self.tags = None
        self.list_usage = None

        super(AzureRMNetworkInterfaceInfo, self).__init__(self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=False,
                                                          facts_module=True,
                                                          required_by=self.required_by)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            results = self.get_item()
        elif self.resource_group is not None:
            results = self.list_resource_group()
        else:
            results = self.list_items()

        self.results['virtualnetworks'] = self.curated(results)

        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.name))
        item = None
        results = []

        try:
            item = self.network_client.virtual_networks.get(resource_group_name=self.resource_group,
                                                            virtual_network_name=self.name)
        except ResourceNotFoundError:
            pass

        if item and self.has_tags(item.tags, self.tags):
            results = [item]
        return results

    def list_resource_group(self):
        self.log('List items for resource group')
        try:
            response = self.network_client.virtual_networks.list(self.resource_group)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list for resource group {0} - {1}".format(self.resource_group, str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(item)
        return results

    def list_items(self):
        self.log('List all for items')
        try:
            response = self.network_client.virtual_networks.list_all()
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items - {0}".format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(item)
        return results

    def serialize(self, raws):
        self.log("Serialize all items")
        return [self.serialize_obj(item, AZURE_OBJECT_CLASS) for item in raws] if raws else []

    def curated(self, raws):
        self.log("Format all items")
        return [self.virtualnetwork_to_dict(x) for x in raws] if raws else []

    def virtualnetwork_to_dict(self, vnet):
        results = dict(
            id=vnet.id,
            name=vnet.name,
            resource_group=parse_resource_id(vnet.id).get('resource_group'),
            location=vnet.location,
            tags=vnet.tags,
            provisioning_state=vnet.provisioning_state,
            flow_timeout_in_minutes=vnet.flow_timeout_in_minutes
        )
        if vnet.dhcp_options and len(vnet.dhcp_options.dns_servers) > 0:
            results['dns_servers'] = []
            for server in vnet.dhcp_options.dns_servers:
                results['dns_servers'].append(server)
        if vnet.address_space and len(vnet.address_space.address_prefixes) > 0:
            results['address_prefixes'] = []
            for space in vnet.address_space.address_prefixes:
                results['address_prefixes'].append(space)
        if vnet.subnets and len(vnet.subnets) > 0:
            results['subnets'] = [self.subnet_to_dict(x) for x in vnet.subnets]
        if self.list_usage:
            results['usages'] = self.list_usages(results['resource_group'], results['name'])

        return results

    def subnet_to_dict(self, subnet):
        result = dict(
            id=subnet.id,
            name=subnet.name,
            provisioning_state=subnet.provisioning_state,
            address_prefix=subnet.address_prefix,
            address_prefixes=subnet.address_prefixes if subnet.address_prefixes else None,
            network_security_group=subnet.network_security_group.id if subnet.network_security_group else None,
            route_table=subnet.route_table.id if subnet.route_table else None
        )
        if subnet.service_endpoints:
            result['service_endpoints'] = [{'service': item.service, 'locations': item.locations} for item in subnet.service_endpoints]
        return result

    def list_usages(self, resource_group, name):
        results = []
        try:
            response = self.network_client.virtual_networks.list_usage(resource_group, name)
        except HttpResponseError as ec:
            self.fail("Faild to list usage status")

        for item in response:
            results.append(item.as_dict())
        return results


def main():
    AzureRMNetworkInterfaceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Daniil Kupchenko (@oukooveu)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: msk_config
short_description: Manage Amazon MSK cluster configurations
version_added: "2.0.0"
description:
    - Create, delete and modify Amazon MSK (Managed Streaming for Apache Kafka) cluster configurations.
    - Prior to release 5.0.0 this module was called C(community.aws.aws_msk_config).
      The usage did not change.
author:
    - Daniil Kupchenko (@oukooveu)
options:
    state:
        description: Create (C(present)) or delete (C(absent)) cluster configuration.
        choices: ['present', 'absent']
        default: 'present'
        type: str
    name:
        description: The name of the configuration.
        required: true
        type: str
    description:
        description: The description of the configuration.
        type: str
        default: ''
    config:
        description: Contents of the server.properties file.
        type: dict
        default: {}
        aliases: ['configuration']
    kafka_versions:
        description:
            - The versions of Apache Kafka with which you can use this MSK configuration.
            - Required when I(state=present).
        type: list
        elements: str
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- community.aws.msk_config:
    name: kafka-cluster-configuration
    state: present
    kafka_versions:
      - 2.6.0
      - 2.6.1
    config:
      auto.create.topics.enable: false
      num.partitions: 1
      default.replication.factor: 3
      zookeeper.session.timeout.ms: 18000

- community.aws.msk_config:
    name: kafka-cluster-configuration
    state: absent
"""

RETURN = r"""
# These are examples of possible return values, and in general should use other names for return values.

arn:
    description: The Amazon Resource Name (ARN) of the configuration.
    type: str
    returned: I(state=present)
    sample: "arn:aws:kafka:<region>:<account>:configuration/<name>/<resource-id>"
revision:
    description: The revision number.
    type: int
    returned: I(state=present)
    sample: 1
server_properties:
    description: Contents of the server.properties file.
    type: str
    returned: I(state=present)
    sample: "default.replication.factor=3\nnum.io.threads=8\nzookeeper.session.timeout.ms=18000"
response:
    description: The response from actual API call.
    type: dict
    returned: always
    sample: {}
"""

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.iam import get_aws_account_info
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def dict_to_prop(d):
    """convert dictionary to multi-line properties"""
    if len(d) == 0:
        return ""
    return "\n".join(f"{k}={v}" for k, v in d.items())


def prop_to_dict(p):
    """convert properties to dictionary"""
    if len(p) == 0:
        return {}
    r_dict = {}
    for s in p.decode().split("\n"):
        kv = s.split("=")
        r_dict[kv[0].strip()] = kv[1].strip()
    return r_dict
    # python >= 2.7 is required:
    # return {
    #     k.strip(): v.strip() for k, v in (i.split("=") for i in p.decode().split("\n"))
    # }


@AWSRetry.jittered_backoff(retries=5, delay=5)
def get_configurations_with_backoff(client):
    paginator = client.get_paginator("list_configurations")
    return paginator.paginate().build_full_result()


def find_active_config(client, module):
    """
    looking for configuration by name
    """

    name = module.params["name"]

    try:
        all_configs = get_configurations_with_backoff(client)["Configurations"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="failed to obtain kafka configurations")

    active_configs = list(item for item in all_configs if item["Name"] == name and item["State"] == "ACTIVE")

    if active_configs:
        if len(active_configs) == 1:
            return active_configs[0]
        else:
            module.fail_json_aws(msg=f"found more than one active config with name '{name}'")

    return None


def get_configuration_revision(client, module, arn, revision):
    try:
        return client.describe_configuration_revision(Arn=arn, Revision=revision, aws_retry=True)
    except (
        botocore.exceptions.BotoCoreError,
        botocore.exceptions.ClientError,
    ) as e:
        module.fail_json_aws(e, "failed to describe kafka configuration revision")


def is_configuration_changed(module, current):
    """
    compare configuration's description and properties
    python 2.7+ version:
    prop_module = {str(k): str(v) for k, v in module.params.get("config").items()}
    """
    prop_module = {}
    for k, v in module.params.get("config").items():
        prop_module[str(k)] = str(v)
    if prop_to_dict(current.get("ServerProperties", "")) == prop_module:
        if current.get("Description", "") == module.params.get("description"):
            return False
    return True


def create_config(client, module):
    """create new or update existing configuration"""

    config = find_active_config(client, module)

    # create new configuration
    if not config:
        if module.check_mode:
            return True, {}

        try:
            response = client.create_configuration(
                Name=module.params.get("name"),
                Description=module.params.get("description"),
                KafkaVersions=module.params.get("kafka_versions"),
                ServerProperties=dict_to_prop(module.params.get("config")).encode(),
                aws_retry=True,
            )
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:
            module.fail_json_aws(e, "failed to create kafka configuration")

    # update existing configuration (creates new revision)
    else:
        # it's required because 'config' doesn't contain 'ServerProperties'
        response = get_configuration_revision(
            client, module, arn=config["Arn"], revision=config["LatestRevision"]["Revision"]
        )

        if not is_configuration_changed(module, response):
            return False, response

        if module.check_mode:
            return True, {}

        try:
            response = client.update_configuration(
                Arn=config["Arn"],
                Description=module.params.get("description"),
                ServerProperties=dict_to_prop(module.params.get("config")).encode(),
                aws_retry=True,
            )
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:
            module.fail_json_aws(e, "failed to update kafka configuration")

    arn = response["Arn"]
    revision = response["LatestRevision"]["Revision"]

    result = get_configuration_revision(client, module, arn=arn, revision=revision)

    return True, result


def delete_config(client, module):
    """delete configuration"""

    config = find_active_config(client, module)

    if module.check_mode:
        if config:
            return True, config
        else:
            return False, {}

    if config:
        try:
            response = client.delete_configuration(Arn=config["Arn"], aws_retry=True)
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:
            module.fail_json_aws(e, "failed to delete the kafka configuration")
        return True, response

    return False, {}


def main():
    module_args = dict(
        name=dict(type="str", required=True),
        description=dict(type="str", default=""),
        state=dict(choices=["present", "absent"], default="present"),
        config=dict(type="dict", aliases=["configuration"], default={}),
        kafka_versions=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=module_args, supports_check_mode=True)

    client = module.client("kafka", retry_decorator=AWSRetry.jittered_backoff())

    if module.params["state"] == "present":
        changed, response = create_config(client, module)

    elif module.params["state"] == "absent":
        changed, response = delete_config(client, module)

    # return some useless staff in check mode if configuration doesn't exists
    # can be useful when these options are referenced by other modules during check mode run
    if module.check_mode and not response.get("Arn"):
        account_id, partition = get_aws_account_info(module)
        arn = f"arn:{partition}:kafka:{module.region}:{account_id}:configuration/{module.params['name']}/id"
        revision = 1
        server_properties = ""
    else:
        arn = response.get("Arn")
        revision = response.get("Revision")
        server_properties = response.get("ServerProperties", "")

    module.exit_json(
        changed=changed,
        arn=arn,
        revision=revision,
        server_properties=server_properties,
        response=camel_dict_to_snake_dict(response),
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ethernet_network_group_policy
short_description: Ethernet Network Group Policy configuration for Cisco Intersight
description:
  - Manages Ethernet Network Group Policy configuration on Cisco Intersight.
  - A policy to configure VLAN settings and QinQ (802.1Q-in-802.1Q) tunneling for Ethernet virtual interfaces on Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/EthNetworkGroupPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
      - When C(absent), VLAN configuration parameters are not required.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Ethernet Network Group Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Ethernet Network Group Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  qinq_enabled:
    description:
      - Enable QinQ (802.1Q-in-802.1Q) Tunneling on the vNIC.
      - When enabled, C(qinq_vlan) is required and C(allowed_vlans) is ignored.
      - When disabled, C(allowed_vlans) is required.
    type: bool
    default: false
  qinq_vlan:
    description:
      - Set QinQ VLAN number.
      - Required when C(qinq_enabled) is true.
      - Only one QinQ VLAN can be specified.
      - Valid range is 2-4093.
    type: int
  native_vlan:
    description:
      - Set native VLAN for the Ethernet Network Group Policy.
      - Can be used with QinQ (when C(qinq_enabled) is true) or with regular VLAN mode (when C(qinq_enabled) is false).
      - When used with C(allowed_vlans), the native VLAN must be included in the allowed VLANs list.
      - Only one native VLAN can be specified.
      - Valid range is 1-4093.
    type: int
  allowed_vlans:
    description:
      - Include VLAN IDs using a list of comma separated VLAN IDs and VLAN ID Ranges.
      - Required when C(qinq_enabled) is false.
      - Examples of valid formats are C(1), C(1,2,3,4,8), C(1-4,7), C(1-8,12,16).
      - Valid VLAN range is 1-4093.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create an Ethernet Network Group Policy with regular VLANs
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "regular-vlans-policy"
    description: "Policy with regular VLAN configuration"
    qinq_enabled: false
    allowed_vlans: "1-8,12,16"
    state: present

- name: Create an Ethernet Network Group Policy with QinQ enabled
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "qinq-policy"
    description: "Policy with QinQ configuration"
    tags:
      - Key: "Environment"
        Value: "Production"
    qinq_enabled: true
    qinq_vlan: 4
    native_vlan: 1
    state: present

- name: Create an Ethernet Network Group Policy with QinQ enabled (no native VLAN)
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "qinq-no-native-policy"
    description: "QinQ policy without native VLAN"
    qinq_enabled: true
    qinq_vlan: 100
    state: present

- name: Create an Ethernet Network Group Policy with single VLAN
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "single-vlan-policy"
    description: "Policy with single VLAN"
    qinq_enabled: false
    allowed_vlans: "50"
    state: present

- name: Create an Ethernet Network Group Policy with native VLAN
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "native-vlan-policy"
    description: "Policy with allowed VLANs and native VLAN"
    qinq_enabled: false
    allowed_vlans: "1-100,200-300"
    native_vlan: 1
    state: present

- name: Delete an Ethernet Network Group Policy
  cisco.intersight.intersight_ethernet_network_group_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "qinq-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "regular-vlans-policy",
        "ObjectType": "fabric.EthNetworkGroupPolicy",
        "VlanSettings": {
            "ClassId": "fabric.VlanSettings",
            "ObjectType": "fabric.VlanSettings",
            "QinqEnabled": false,
            "AllowedVlans": "1-5,10,15-20"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Prod"
            }
        ]
    }
'''


import re
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_allowed_vlans_format(allowed_vlans):
    """Validate allowed_vlans format and return parsed VLAN IDs"""
    if not allowed_vlans or not allowed_vlans.strip():
        return False, "allowed_vlans cannot be empty"

    # Remove all whitespace
    vlans_str = allowed_vlans.replace(' ', '')

    # Pattern to match: single VLAN (123) or VLAN range (123-456)
    # Full pattern: comma-separated list of single VLANs or ranges
    pattern = r'^(\d+(-\d+)?)(,\d+(-\d+)?)*$'

    if not re.match(pattern, vlans_str):
        return False, "allowed_vlans must be in format '1', '1,2,3', '1-4', or '1-4,7,10-15'"

    # Parse and validate individual VLAN IDs and ranges
    vlan_parts = vlans_str.split(',')
    for part in vlan_parts:
        if '-' in part:
            try:
                start, end = part.split('-')
                start_id, end_id = int(start), int(end)
                if start_id >= end_id:
                    return False, f"Invalid VLAN range '{part}': start must be less than end"
                if start_id < 1 or start_id > 4093 or end_id < 1 or end_id > 4093:
                    return False, f"VLAN IDs in range '{part}' must be between 1 and 4093"
            except ValueError:
                return False, f"Invalid VLAN range format '{part}'"
        else:
            # Validate single vlan
            try:
                vlan_id = int(part)
                if vlan_id < 1 or vlan_id > 4093:
                    return False, f"VLAN ID '{vlan_id}' must be between 1 and 4093"
            except ValueError:
                return False, f"Invalid VLAN ID '{part}'"

    return True, None


def is_vlan_in_allowed_vlans(vlan_id, allowed_vlans_str):
    """Check if a VLAN ID is included in the allowed VLANs string"""
    # Remove all whitespace
    vlans_str = allowed_vlans_str.replace(' ', '')
    vlan_parts = vlans_str.split(',')

    for part in vlan_parts:
        if '-' in part:
            # VLAN range
            start, end = part.split('-')
            start_id, end_id = int(start), int(end)
            if start_id <= vlan_id <= end_id:
                return True
        else:
            # Single VLAN
            if int(part) == vlan_id:
                return True

    return False


def validate_vlan_ranges(module):
    """Validate VLAN range values and required parameters"""
    state = module.params.get('state')
    qinq_enabled = module.params.get('qinq_enabled')
    qinq_vlan = module.params.get('qinq_vlan')
    native_vlan = module.params.get('native_vlan')
    allowed_vlans = module.params.get('allowed_vlans')

    if state == 'present':
        if qinq_enabled:
            # When qinq_enabled is true, qinq_vlan is required
            if not qinq_vlan:
                module.fail_json(msg='missing required arguments: qinq_vlan')
        else:
            # When qinq_enabled is false, allowed_vlans is required
            if not allowed_vlans:
                module.fail_json(msg='missing required arguments: allowed_vlans')

    if qinq_vlan and (qinq_vlan < 2 or qinq_vlan > 4093):
        module.fail_json(msg='qinq_vlan must be between 2 and 4093')

    if native_vlan and (native_vlan < 1 or native_vlan > 4093):
        module.fail_json(msg='native_vlan must be between 1 and 4093')

    # Validate allowed_vlans format if provided and clean it
    if allowed_vlans:
        is_valid, error_msg = validate_allowed_vlans_format(allowed_vlans)
        if not is_valid:
            module.fail_json(msg=error_msg)
        # Store the cleaned version (whitespace removed) back to module.params
        module.params['allowed_vlans'] = allowed_vlans.replace(' ', '')

    # Validate native_vlan is in allowed_vlans when both are specified
    if native_vlan and allowed_vlans and not qinq_enabled:
        if not is_vlan_in_allowed_vlans(native_vlan, allowed_vlans):
            module.fail_json(msg=f'native_vlan {native_vlan} must be included in allowed_vlans: {allowed_vlans}')


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        qinq_enabled=dict(type='bool', default=False),  # Verify that placing default works with required_if
        qinq_vlan=dict(type='int'),
        native_vlan=dict(type='int'),
        allowed_vlans=dict(type='str'),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ['allowed_vlans', 'qinq_vlan'],
        ],
    )

    validate_vlan_ranges(module)
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/fabric/EthNetworkGroupPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Build VlanSettings based on qinq_enabled
        vlan_settings = {
            'QinqEnabled': module.params['qinq_enabled']
        }

        if module.params['qinq_enabled']:
            vlan_settings['QinqVlan'] = module.params['qinq_vlan']
            if module.params['native_vlan']:
                vlan_settings['NativeVlan'] = module.params['native_vlan']
        else:
            # Regular VLAN mode
            vlan_settings['AllowedVlans'] = module.params['allowed_vlans']
            if module.params['native_vlan']:
                vlan_settings['NativeVlan'] = module.params['native_vlan']

        intersight.api_body.update({
            'VlanSettings': vlan_settings
        })

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import typing as t
from collections.abc import Mapping, Set

from ansible.module_utils.common.collections import is_sequence
from ansible.utils.unsafe_proxy import (
    AnsibleUnsafe,
)
from ansible.utils.unsafe_proxy import wrap_var as _make_unsafe

_RE_TEMPLATE_CHARS = re.compile("[{}]")
_RE_TEMPLATE_CHARS_BYTES = re.compile(b"[{}]")


def make_unsafe(value: t.Any) -> t.Any:
    if value is None or isinstance(value, AnsibleUnsafe):
        return value

    if isinstance(value, Mapping):
        return dict((make_unsafe(key), make_unsafe(val)) for key, val in value.items())
    if isinstance(value, Set):
        return set(make_unsafe(elt) for elt in value)
    if is_sequence(value):
        return type(value)(make_unsafe(elt) for elt in value)
    if isinstance(value, bytes):
        if _RE_TEMPLATE_CHARS_BYTES.search(value):
            value = _make_unsafe(value)
        return value
    if isinstance(value, str):
        if _RE_TEMPLATE_CHARS.search(value):
            value = _make_unsafe(value)
        return value

    return value



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c) 2016, Adfinis SyGroup AG
# @keachi
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

"""Univention Corporate Server (UCS) access module.

Provides the following functions for working with an UCS server.

  - ldap_search(filter, base=None, attr=None)
    Search the LDAP via Univention's LDAP wrapper (ULDAP)

  - config_registry()
    Return the UCR registry object

  - base_dn()
    Return the configured Base DN according to the UCR

  - uldap()
    Return a handle to the ULDAP LDAP wrapper

  - umc_module_for_add(module, container_dn, superordinate=None)
    Return a UMC module for creating a new object of the given type

  - umc_module_for_edit(module, object_dn, superordinate=None)
    Return a UMC module for editing an existing object of the given type


Any other module is not part of the "official" API and may change at any time.
"""

import re

__all__ = [
    "base_dn",
    "config_registry",
    "ldap_search",
    "uldap",
    "umc_module_for_add",
    "umc_module_for_edit",
]


_singletons: dict[str, object] = {}


def ldap_module():
    import ldap as orig_ldap

    return orig_ldap


def _singleton(name: str, constructor):
    if name in _singletons:
        return _singletons[name]
    _singletons[name] = constructor()
    return _singletons[name]


def config_registry():
    def construct():
        import univention.config_registry

        ucr = univention.config_registry.ConfigRegistry()
        ucr.load()
        return ucr

    return _singleton("config_registry", construct)


def base_dn():
    return config_registry()["ldap/base"]


def uldap():
    "Return a configured univention uldap object"

    def construct():
        try:
            secret_file = open("/etc/ldap.secret")
            bind_dn = f"cn=admin,{base_dn()}"
        except OSError:  # pragma: no cover
            secret_file = open("/etc/machine.secret")
            bind_dn = config_registry()["ldap/hostdn"]
        pwd_line = secret_file.readline()
        pwd = re.sub("\n", "", pwd_line)

        import univention.admin.uldap

        return univention.admin.uldap.access(
            host=config_registry()["ldap/master"],
            base=base_dn(),
            binddn=bind_dn,
            bindpw=pwd,
            start_tls=1,
        )

    return _singleton("uldap", construct)


def config():
    def construct():
        import univention.admin.config

        return univention.admin.config.config()

    return _singleton("config", construct)


def init_modules():
    def construct():
        import univention.admin.modules

        univention.admin.modules.update()
        return True

    return _singleton("modules_initialized", construct)


def position_base_dn():
    def construct():
        import univention.admin.uldap

        return univention.admin.uldap.position(base_dn())

    return _singleton("position_base_dn", construct)


def ldap_dn_tree_parent(dn, count=1):
    dn_array = dn.split(",")
    dn_array[0:count] = []
    return ",".join(dn_array)


def ldap_search(filter, base=None, attr=None):
    """Replaces uldaps search and uses a generator.
    !! Arguments are not the same."""

    if base is None:
        base = base_dn()
    msgid = uldap().lo.lo.search(base, ldap_module().SCOPE_SUBTREE, filterstr=filter, attrlist=attr)
    # I used to have a try: finally: here but there seems to be a bug in python
    # which swallows the KeyboardInterrupt
    # The abandon now doesn't make too much sense
    while True:
        result_type, result_data = uldap().lo.lo.result(msgid, all=0)
        if not result_data:
            break
        if result_type is ldap_module().RES_SEARCH_RESULT:  # pragma: no cover
            break
        else:
            if result_type is ldap_module().RES_SEARCH_ENTRY:
                yield from result_data
    uldap().lo.lo.abandon(msgid)


def module_by_name(module_name_: str):
    """Returns an initialized UMC module, identified by the given name.

    The module is a module specification according to the udm commandline.
    Example values are:
    * users/user
    * shares/share
    * groups/group

    If the module does not exist, a KeyError is raised.

    The modules are cached, so they won't be re-initialized
    in subsequent calls.
    """

    def construct():
        import univention.admin.modules

        init_modules()
        module = univention.admin.modules.get(module_name_)
        univention.admin.modules.init(uldap(), position_base_dn(), module)
        return module

    return _singleton(f"module/{module_name_}", construct)


def get_umc_admin_objects():
    """Convenience accessor for getting univention.admin.objects.

    This implements delayed importing, so the univention.* modules
    are not loaded until this function is called.
    """
    import univention.admin

    return univention.admin.objects


def umc_module_for_add(module: str, container_dn, superordinate=None):
    """Returns an UMC module object prepared for creating a new entry.

    The module is a module specification according to the udm commandline.
    Example values are:
        * users/user
        * shares/share
        * groups/group

    The container_dn MUST be the dn of the container (not of the object to
    be created itself!).
    """
    mod = module_by_name(module)

    position = position_base_dn()
    position.setDn(container_dn)

    # config, ldap objects from common module
    obj = mod.object(config(), uldap(), position, superordinate=superordinate)
    obj.open()

    return obj


def umc_module_for_edit(module: str, object_dn, superordinate=None):
    """Returns an UMC module object prepared for editing an existing entry.

    The module is a module specification according to the udm commandline.
    Example values are:
        * users/user
        * shares/share
        * groups/group

    The object_dn MUST be the dn of the object itself, not the container!
    """
    mod = module_by_name(module)

    objects = get_umc_admin_objects()

    position = position_base_dn()
    position.setDn(ldap_dn_tree_parent(object_dn))

    obj = objects.get(mod, config(), uldap(), position=position, superordinate=superordinate, dn=object_dn)
    obj.open()

    return obj


def create_containers_and_parents(container_dn):
    """Create a container and if needed the parents containers"""
    import univention.admin.uexceptions as uexcp

    if not container_dn.startswith("cn="):
        raise AssertionError()
    try:
        parent = ldap_dn_tree_parent(container_dn)
        obj = umc_module_for_add("container/cn", parent)
        obj["name"] = container_dn.split(",")[0].split("=")[1]
        obj["description"] = "container created by import"
    except uexcp.ldapError:
        create_containers_and_parents(parent)
        obj = umc_module_for_add("container/cn", parent)
        obj["name"] = container_dn.split(",")[0].split("=")[1]
        obj["description"] = "container created by import"



# -*- coding: utf-8 -*-

# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: aws_rds
short_description: RDS instance inventory source
description:
  - Get instances and clusters from Amazon Web Services RDS.
  - Uses a YAML configuration file that ends with aws_rds.(yml|yaml).
options:
  regions:
    description:
      - A list of regions in which to describe RDS instances and clusters. Available regions are listed here
        U(https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.RegionsAndAvailabilityZones.html).
    default: []
  filters:
    description:
      - A dictionary of filter value pairs. Available filters are listed here
        U(https://docs.aws.amazon.com/cli/latest/reference/rds/describe-db-instances.html#options). If you filter by
        db-cluster-id and I(include_clusters) is True it will apply to clusters as well.
    default: {}
  strict_permissions:
    description:
      - By default if an AccessDenied exception is encountered this plugin will fail. You can set strict_permissions to
        False in the inventory config file which will allow the restrictions to be gracefully skipped.
    type: bool
    default: true
  include_clusters:
    description: Whether or not to query for Aurora clusters as well as instances.
    type: bool
    default: false
  statuses:
    description: A list of desired states for instances/clusters to be added to inventory. Set to ['all'] as a shorthand to find everything.
    type: list
    elements: str
    default:
      - creating
      - available
  hostvars_prefix:
    description:
      - The prefix for host variables names coming from AWS.
    type: str
    version_added: 3.1.0
  hostvars_suffix:
    description:
      - The suffix for host variables names coming from AWS.
    type: str
    version_added: 3.1.0
notes:
  - Ansible versions prior to 2.10 should use the fully qualified plugin name 'amazon.aws.aws_rds'.
  - The C(tags) host variable is deprecated and will be removed in a release after 2026-12-01.
    Use C(rds_tags) instead to avoid conflicts with Ansible reserved variable names.
  - The C(rds_tags) host variable was added in version 11.2.0.
extends_documentation_fragment:
  - inventory_cache
  - constructed
  - amazon.aws.boto3
  - amazon.aws.common.plugins
  - amazon.aws.region.plugins
  - amazon.aws.assume_role.plugins
author:
  - Sloane Hertel (@s-hertel)
"""

EXAMPLES = r"""
plugin: aws_rds
regions:
  - us-east-1
  - ca-central-1
keyed_groups:
  - key: 'db_parameter_groups|json_query("[].db_parameter_group_name")'
    prefix: rds_parameter_group
  - key: engine
    prefix: rds
  - key: rds_tags
  - key: region
hostvars_prefix: aws_
hostvars_suffix: _rds
"""

try:
    import botocore
except ImportError:
    pass  # will be captured by imported HAS_BOTO3

from ansible.errors import AnsibleError
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.plugin_utils.inventory import AWSInventoryBase


def _find_hosts_with_valid_statuses(hosts, statuses):
    if "all" in statuses:
        return hosts
    valid_hosts = []
    for host in hosts:
        if host.get("DBInstanceStatus") in statuses:
            valid_hosts.append(host)
        elif host.get("Status") in statuses:
            valid_hosts.append(host)
    return valid_hosts


def _get_rds_hostname(host):
    if host.get("DBInstanceIdentifier"):
        return host["DBInstanceIdentifier"]
    else:
        return host["DBClusterIdentifier"]


def _add_tags_for_rds_hosts(connection, hosts, strict):
    for host in hosts:
        if "DBInstanceArn" in host:
            resource_arn = host["DBInstanceArn"]
        else:
            resource_arn = host["DBClusterArn"]

        try:
            tags = connection.list_tags_for_resource(ResourceName=resource_arn)["TagList"]
        except is_boto3_error_code("AccessDenied") as e:
            if not strict:
                tags = []
            else:
                raise e
        host["Tags"] = tags


def describe_resource_with_tags(func):
    def describe_wrapper(connection, filters, strict=False):
        try:
            results = func(connection=connection, filters=filters)
            if "DBInstances" in results:
                results = results["DBInstances"]
            else:
                results = results["DBClusters"]
            _add_tags_for_rds_hosts(connection, results, strict)
        except is_boto3_error_code("AccessDenied") as e:  # pylint: disable=duplicate-except
            if not strict:
                return []
            raise AnsibleError(f"Failed to query RDS: {to_native(e)}")
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:  # pylint: disable=duplicate-except
            raise AnsibleError(f"Failed to query RDS: {to_native(e)}")

        return results

    return describe_wrapper


@describe_resource_with_tags
def _describe_db_instances(connection, filters):
    paginator = connection.get_paginator("describe_db_instances")
    return paginator.paginate(Filters=filters).build_full_result()


@describe_resource_with_tags
def _describe_db_clusters(connection, filters):
    return connection.describe_db_clusters(Filters=filters)


class InventoryModule(AWSInventoryBase):
    NAME = "amazon.aws.aws_rds"
    INVENTORY_FILE_SUFFIXES = ("aws_rds.yml", "aws_rds.yaml")

    def __init__(self):
        super().__init__()
        self.credentials = {}

    def _populate(self, hosts):
        group = "aws_rds"
        self.inventory.add_group(group)
        if hosts:
            self._add_hosts(hosts=hosts, group=group)
            self.inventory.add_child("all", group)

    def _populate_from_source(self, source_data):
        hostvars = source_data.pop("_meta", {}).get("hostvars", {})
        for group in source_data:
            if group == "all":
                continue
            self.inventory.add_group(group)
            hosts = source_data[group].get("hosts", [])
            for host in hosts:
                self._populate_host_vars([host], hostvars.get(host, {}), group)
            self.inventory.add_child("all", group)

    def _format_inventory(self, hosts):
        results = {"_meta": {"hostvars": {}}}
        group = "aws_rds"
        results[group] = {"hosts": []}
        for host in hosts:
            hostname = _get_rds_hostname(host)
            results[group]["hosts"].append(hostname)
            h = self.inventory.get_host(hostname)
            results["_meta"]["hostvars"][h.name] = h.vars
        return results

    def _add_hosts(self, hosts, group):
        """
        :param hosts: a list of hosts to be added to a group
        :param group: the name of the group to which the hosts belong
        """
        for host in hosts:
            hostname = _get_rds_hostname(host)
            host = camel_dict_to_snake_dict(host, ignore_list=["Tags"])
            host["rds_tags"] = boto3_tag_list_to_ansible_dict(host.get("tags", []))
            # rds_tags is the new key, tags is deprecated but kept for backward compatibility
            host["tags"] = host["rds_tags"]

            # Allow easier grouping by region
            if "availability_zone" in host:
                host["region"] = host["availability_zone"][:-1]
            elif "availability_zones" in host:
                host["region"] = host["availability_zones"][0][:-1]

            self.inventory.add_host(hostname, group=group)
            hostvars_prefix = self.get_option("hostvars_prefix")
            hostvars_suffix = self.get_option("hostvars_suffix")
            new_vars = dict()
            for hostvar, hostval in host.items():
                if hostvars_prefix:
                    hostvar = hostvars_prefix + hostvar
                if hostvars_suffix:
                    hostvar = hostvar + hostvars_suffix
                new_vars[hostvar] = hostval
                self.inventory.set_variable(hostname, hostvar, hostval)
            host.update(new_vars)

            # Use constructed if applicable
            strict = self.get_option("strict")
            # Composed variables
            self._set_composite_vars(self.get_option("compose"), host, hostname, strict=strict)
            # Complex groups based on jinja2 conditionals, hosts that meet the conditional are added to group
            self._add_host_to_composed_groups(self.get_option("groups"), host, hostname, strict=strict)
            # Create groups based on variable values and add the corresponding hosts to it
            self._add_host_to_keyed_groups(self.get_option("keyed_groups"), host, hostname, strict=strict)

    def _get_all_db_hosts(self, regions, instance_filters, cluster_filters, strict, statuses, gather_clusters=False):
        """
        :param regions: a list of regions in which to describe hosts
        :param instance_filters: a list of boto3 filter dictionaries
        :param cluster_filters: a list of boto3 filter dictionaries
        :param strict: a boolean determining whether to fail or ignore 403 error codes
        :param statuses: a list of statuses that the returned hosts should match
        :return A list of host dictionaries
        """
        all_instances = []
        all_clusters = []

        for connection, _region in self.all_clients("rds"):
            all_instances += _describe_db_instances(connection, instance_filters, strict=strict)
            if gather_clusters:
                all_clusters += _describe_db_clusters(connection, cluster_filters, strict=strict)
        sorted_hosts = list(
            sorted(all_instances, key=lambda x: x["DBInstanceIdentifier"])
            + sorted(all_clusters, key=lambda x: x["DBClusterIdentifier"])
        )
        return _find_hosts_with_valid_statuses(sorted_hosts, statuses)

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path, cache=cache)

        self.display.deprecated(
            "The 'tags' host variable is deprecated. Use 'rds_tags' instead.",
            date="2026-12-01",
            collection_name="amazon.aws",
        )

        # get user specifications
        regions = self.get_option("regions")
        filters = self.get_option("filters")
        strict_permissions = self.get_option("strict_permissions")
        statuses = self.get_option("statuses")
        include_clusters = self.get_option("include_clusters")
        instance_filters = ansible_dict_to_boto3_filter_list(filters)
        cluster_filters = []
        if "db-cluster-id" in filters and include_clusters:
            cluster_filters = ansible_dict_to_boto3_filter_list({"db-cluster-id": filters["db-cluster-id"]})

        result_was_cached, cached_result = self.get_cached_result(path, cache)
        if result_was_cached:
            self._populate_from_source(cached_result)
            return

        results = self._get_all_db_hosts(
            regions,
            instance_filters,
            cluster_filters,
            strict_permissions,
            statuses,
            include_clusters,
        )
        self._populate(results)

        # Update the cache once we're done
        formatted_inventory = self._format_inventory(results)
        self.update_cached_result(path, cache, formatted_inventory)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for nxos_lacp_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_lacp_interfaces
short_description: LACP interfaces resource module
description: This module manages Link Aggregation Control Protocol (LACP) attributes
  of NX-OS Interfaces.
version_added: 1.0.0
author: Trishna Guha (@trishnaguha)
notes:
- Tested against NXOS 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section ^interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A dictionary of LACP interfaces options.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name of the interface.
        required: true
        type: str
      port_priority:
        description:
        - LACP port priority for the interface. Range 1-65535. Applicable only for
          Ethernet.
        type: int
      rate:
        description:
        - Rate at which PDUs are sent by LACP. Applicable only for Ethernet. At fast
          rate LACP is transmitted once every 1 second. At normal rate LACP is transmitted
          every 30 seconds after the link is bundled.
        type: str
        choices:
        - fast
        - normal
      links:
        description:
        - This dict contains configurable options related to max and min port-channel
          links. Applicable only for Port-channel.
        type: dict
        suboptions:
          max:
            description:
            - Port-channel max bundle.
            type: int
          min:
            description:
            - Port-channel min links.
            type: int
      mode:
        description:
        - LACP mode. Applicable only for Port-channel.
        type: str
        choices:
        - delay
      suspend_individual:
        description:
        - port-channel lacp state. Disabling this will cause lacp to put the port
          to individual state and not suspend the port in case it does not get LACP
          BPDU from the peer ports in the port-channel.
        type: bool
      convergence:
        description:
        - This dict contains configurable options related to convergence. Applicable
          only for Port-channel.
        type: dict
        suboptions:
          graceful:
            description:
            - port-channel lacp graceful convergence. Disable this only with lacp
              ports connected to Non-Nexus peer. Disabling this with Nexus peer can
              lead to port suspension.
            type: bool
          vpc:
            description:
            - Enable lacp convergence for vPC port channels.
            type: bool
  state:
    description:
    - The state of the configuration after module completion.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - rendered
    - parsed
    default: merged

"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#

- name: Merge provided configuration with device configuration.
  cisco.nxos.nxos_lacp_interfaces:
    config:
      - name: Ethernet1/3
        port_priority: 5
        rate: fast
    state: merged

# After state:
# ------------
#
# interface Ethernet1/3
# lacp port-priority 5
# lacp rate fast


# Using replaced

# Before state:
# -------------
#
# interface Ethernet1/3
#   lacp port-priority 5
# interface port-channel11
#   lacp mode delay

- name: Replace device lacp interfaces configuration with the given configuration.
  cisco.nxos.nxos_lacp_interfaces:
    config:
      - name: port-channel11
        links:
          min: 4
    state: replaced

# After state:
# ------------
#
# interface Ethernet1/3
#   lacp port-priority 5
# interface port-channel11
#   lacp min-links 4


# Using overridden

# Before state:
# -------------
#
# interface Ethernet1/3
#   lacp port-priority 5
# interface port-channel11
#   lacp mode delay

- name: Override device configuration of all LACP interfaces attributes of given interfaces
    on device with provided configuration.
  cisco.nxos.nxos_lacp_interfaces:
    config:
      - name: port-channel11
        links:
          min: 4
    state: overridden

# After state:
# ------------
#
# interface port-channel11
# lacp min-links 4


# Using deleted

# Before state:
# -------------
#
# interface Ethernet1/3
#   lacp port-priority 5
# interface port-channel11
#   lacp mode delay

- name: Delete LACP interfaces configurations.
  cisco.nxos.nxos_lacp_interfaces:
    state: deleted

# After state:
# ------------
#

# Using rendered

- name: Use rendered state to convert task input to device specific commands
  cisco.nxos.nxos_lacp_interfaces:
    config:
      - name: Ethernet1/800
        rate: fast
      - name: Ethernet1/801
        rate: fast
        port_priority: 32
      - name: port-channel10
        links:
          max: 15
          min: 2
        convergence:
          graceful: true
    state: rendered

# Task Output (redacted)
# -----------------------

# rendered:
#  - "interface Ethernet1/800"
#  - "lacp rate fast"
#  - "interface Ethernet1/801"
#  - "lacp port-priority 32"
#  - "lacp rate fast"
#  - "interface port-channel10"
#  - "lacp min-links 2"
#  - "lacp max-bundle 15"
#  - "lacp graceful-convergence"

# Using parsed

# parsed.cfg
# ------------

# interface port-channel10
#   lacp min-links 10
#   lacp max-bundle 15
# interface Ethernet1/800
#   lacp port-priority 100
#   lacp rate fast

- name: Use parsed state to convert externally supplied config to structured format
  cisco.nxos.nxos_lacp_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output (redacted)
# -----------------------

# parsed:
#   - name: port-channel10
#     links:
#       max: 15
#       min: 10
#   - name: Ethernet1/800
#     port_priority: 100
#     rate: fast

# Using gathered

# Existing device config state
# -------------------------------
# interface Ethernet1/1
#   lacp port-priority 5
#   lacp rate fast
# interface port-channel10
#   lacp mode delay
# interface port-channel11
#   lacp max-bundle 10
#   lacp min-links 5

- name: Gather lacp_interfaces facts from the device using nxos_lacp_interfaces
  cisco.nxos.nxos_lacp_interfaces:
    state: gathered

# Task output (redacted)
# -----------------------
# gathered:
#  - name: Ethernet1/1
#    port_priority: 5
#    rate: fast
#  - name: port-channel10
#    mode: delay
#  - name: port-channel11
#    links:
#      max: 10
#      min: 5
"""
RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface port-channel10', 'lacp min-links 5', 'lacp mode delay']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.lacp_interfaces.lacp_interfaces import (
    Lacp_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.lacp_interfaces.lacp_interfaces import (
    Lacp_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Lacp_interfacesArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    result = Lacp_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_network_probe
short_description: Manages network-probe objects on Checkpoint over Web Services API
description:
  - Manages network-probe objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  http_options:
    description:
      - Additional options when [protocol] is set to "http".
    type: dict
    suboptions:
      destination:
        description:
          - The destination URL.
        type: str
  icmp_options:
    description:
      - Additional options when [protocol] is set to "icmp".
    type: dict
    suboptions:
      destination:
        description:
          - One of these,<br>- Name or UID of an existing object with a unicast IPv4 address (Host, Security Gateway, and so on).<br>- A unicast
            IPv4 address string (if you do not want to create such an object).
        type: str
      source:
        description:
          - One of these,<br>- The string "main-ip" (the probe uses the main IPv4 address of the Security Gateway objects you specified in the
            parameter [install-on]).<br>- Name or UID of an existing object of type 'Host' with a unicast IPv4 address.<br>- A unicast IPv4 address string (if
            you do not want to create such an object).
        type: str
  install_on:
    description:
      - Collection of Check Point Security Gateways that generate the probe, identified by name or UID.
    type: list
    elements: str
  protocol:
    description:
      - The probing protocol to use.
    type: str
    choices: ['http', 'icmp']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  interval:
    description:
      - The time interval in the range of 5-300 (seconds) between each probe request.
        Best Practice - The interval value should be lower than the timeout value.
    type: int
  timeout:
    description:
      - The probe expiration timeout in the range of 5-300 (seconds).
        If there is not a single reply within this time, the status of the probe changes to "Down".
    type: int
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-network-probe
  cp_mgmt_network_probe:
    icmp_options:
      destination: HOST_20.20.20.20
      source: Host_10.10.10.10
    install_on: GW_1
    name: probe_GW1
    state: present

- name: set-network-probe
  cp_mgmt_network_probe:
    icmp_options:
      destination: 2.2.2.2
      source: 1.1.1.1
    name: probe_GW1
    state: present

- name: delete-network-probe
  cp_mgmt_network_probe:
    name: probe_GW1
    state: absent
"""

RETURN = """
cp_mgmt_network_probe:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        http_options=dict(type='dict', options=dict(
            destination=dict(type='str')
        )),
        icmp_options=dict(type='dict', options=dict(
            destination=dict(type='str'),
            source=dict(type='str')
        )),
        install_on=dict(type='list', elements='str'),
        protocol=dict(type='str', choices=['http', 'icmp']),
        tags=dict(type='list', elements='str'),
        interval=dict(type='int'),
        timeout=dict(type='int'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'network-probe'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) 2023, Steffen Scheib <steffen@scheib.me>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: from_ini
short_description: Converts INI text input into a dictionary
version_added: 8.2.0
author: Steffen Scheib (@sscheib)
description:
  - Converts INI text input into a dictionary.
options:
  _input:
    description: A string containing an INI document.
    type: string
    required: true
  delimiters:
    description: A list of characters used as delimiters in the INI document.
    type: list
    elements: string
    default:
      - "="
      - ":"
    version_added: 12.4.0
seealso:
  - plugin: community.general.to_ini
    plugin_type: filter
  - plugin: ansible.builtin.ini
    plugin_type: lookup
  - module: community.general.ini_file
"""

EXAMPLES = r"""
- name: Slurp an INI file
  ansible.builtin.slurp:
    src: /etc/rhsm/rhsm.conf
  register: rhsm_conf

- name: Display the INI file as dictionary
  ansible.builtin.debug:
    var: rhsm_conf.content | b64decode | community.general.from_ini

- name: Set a new dictionary fact with the contents of the INI file
  ansible.builtin.set_fact:
    rhsm_dict: >-
      {{
          rhsm_conf.content | b64decode | community.general.from_ini
      }}
"""

RETURN = r"""
_value:
  description: A dictionary representing the INI file.
  type: dictionary
"""


from configparser import ConfigParser
from io import StringIO

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.collections import is_sequence


class IniParser(ConfigParser):
    """Implements a configparser which is able to return a dict"""

    def __init__(self, delimiters=None):
        if delimiters is None:
            super().__init__(interpolation=None)
        else:
            super().__init__(interpolation=None, delimiters=delimiters)
        self.optionxform = str

    def as_dict(self):
        d = dict(self._sections)
        for k in d:
            d[k] = dict(self._defaults, **d[k])
            d[k].pop("__name__", None)

        if self._defaults:
            d["DEFAULT"] = dict(self._defaults)

        return d


def from_ini(obj, delimiters=None):
    """Read the given string as INI file and return a dict"""

    if not isinstance(obj, str):
        raise AnsibleFilterError(f"from_ini requires a str, got {type(obj)}")
    if delimiters is not None:
        if not is_sequence(delimiters):
            raise AnsibleFilterError(f"from_ini's delimiters parameter must be a sequence, got {type(delimiters)}")
        delimiters = tuple(delimiters)
        if not all(isinstance(elt, str) for elt in delimiters):
            raise AnsibleFilterError(
                f"from_ini's delimiters parameter must be a sequence of strings, got {delimiters!r}"
            )

    parser = IniParser(delimiters=delimiters)

    try:
        parser.read_file(StringIO(obj))
    except Exception as ex:
        raise AnsibleFilterError(f"from_ini failed to parse given string: {ex}", orig_exc=ex) from ex

    return parser.as_dict()


class FilterModule:
    """Query filter"""

    def filters(self):
        return {"from_ini": from_ini}



#!/usr/bin/python
# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_privatekey_info
short_description: Provide information for OpenSSL private keys
description:
  - This module allows one to query information on OpenSSL private keys.
  - In case the key consistency checks fail, the module will fail as this indicates a faked private key. In this case, all
    return variables are still returned. Note that key consistency checks are not available all key types; if none is available,
    V(none) is returned for RV(key_is_consistent).
  - It uses the cryptography python library to interact with OpenSSL.
author:
  - Felix Fontein (@felixfontein)
  - Yanis Guenane (@Spredzy)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._attributes.info_module
  - community.crypto._attributes.idempotent_not_modify_state
  - community.crypto._cryptography_dep.minimum
options:
  path:
    description:
      - Remote absolute path where the private key file is loaded from.
    type: path
  content:
    description:
      - Content of the private key file.
      - Either O(path) or O(content) must be specified, but not both.
    type: str
    version_added: '1.0.0'
  passphrase:
    description:
      - The passphrase for the private key.
    type: str
  return_private_key_data:
    description:
      - Whether to return private key data.
      - Only set this to V(true) when you want private information about this key to leave the remote machine.
      - B(WARNING:) you have to make sure that private key data is not accidentally logged!
    type: bool
    default: false
  check_consistency:
    description:
      - Whether to check consistency of the private key.
      - In community.crypto < 2.0.0, consistency was always checked.
      - Since community.crypto 2.0.0, the consistency check has been disabled by default to avoid private key material to
        be transported around and computed with, and only do so when requested explicitly. This can potentially prevent L(side-channel
        attacks,https://en.wikipedia.org/wiki/Side-channel_attack).
      - Note that consistency checks only work for certain key types, and might depend on the version of the cryptography
        library. For example, with cryptography 42.0.0 and newer consistency of RSA keys can no longer be checked.
    type: bool
    default: false
    version_added: 2.0.0

  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]

seealso:
  - module: community.crypto.openssl_privatekey
  - module: community.crypto.openssl_privatekey_pipe
  - plugin: community.crypto.openssl_privatekey_info
    plugin_type: filter
    description: A filter variant of this module.
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSL private key with the default values (4096 bits, RSA)
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem

- name: Get information on generated key
  community.crypto.openssl_privatekey_info:
    path: /etc/ssl/private/ansible.com.pem
  register: result

- name: Dump information
  ansible.builtin.debug:
    var: result
"""

RETURN = r"""
can_load_key:
  description: Whether the module was able to load the private key from disk.
  returned: always
  type: bool
can_parse_key:
  description: Whether the module was able to parse the private key.
  returned: always
  type: bool
key_is_consistent:
  description:
    - Whether the key is consistent. Can also return V(none) next to V(true) and V(false), to indicate that consistency could
      not be checked.
    - In case the check returns V(false), the module will fail.
  returned: when O(check_consistency=true)
  type: bool
public_key:
  description: Private key's public key in PEM format.
  returned: success
  type: str
  sample: "-----BEGIN PUBLIC KEY-----\nMIICIjANBgkqhkiG9w0BAQEFAAOCAg8A..."
public_key_fingerprints:
  description:
    - Fingerprints of private key's public key.
    - For every hash algorithm available, the fingerprint is computed.
  returned: success
  type: dict
  sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63', 'sha512':
    'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
type:
  description:
    - The key's type.
    - One of V(RSA), V(DSA), V(ECC), V(Ed25519), V(X25519), V(Ed448), or V(X448).
    - Will start with V(unknown) if the key type cannot be determined.
  returned: success
  type: str
  sample: RSA
public_data:
  description:
    - Public key data. Depends on key type.
  returned: success
  type: dict
  contains:
    size:
      description:
        - Bit size of modulus (RSA) or prime number (DSA).
      type: int
      returned: When RV(type=RSA) or RV(type=DSA)
    modulus:
      description:
        - The RSA key's modulus.
      type: int
      returned: When RV(type=RSA)
    exponent:
      description:
        - The RSA key's public exponent.
      type: int
      returned: When RV(type=RSA)
    p:
      description:
        - The C(p) value for DSA.
        - This is the prime modulus upon which arithmetic takes place.
      type: int
      returned: When RV(type=DSA)
    q:
      description:
        - The C(q) value for DSA.
        - This is a prime that divides C(p - 1), and at the same time the order of the subgroup of the multiplicative group
          of the prime field used.
      type: int
      returned: When RV(type=DSA)
    g:
      description:
        - The C(g) value for DSA.
        - This is the element spanning the subgroup of the multiplicative group of the prime field used.
      type: int
      returned: When RV(type=DSA)
    curve:
      description:
        - The curve's name for ECC.
      type: str
      returned: When RV(type=ECC)
    exponent_size:
      description:
        - The maximum number of bits of a private key. This is basically the bit size of the subgroup used.
      type: int
      returned: When RV(type=ECC)
    x:
      description:
        - The C(x) coordinate for the public point on the elliptic curve.
      type: int
      returned: When RV(type=ECC)
    y:
      description:
        - For RV(type=ECC), this is the C(y) coordinate for the public point on the elliptic curve.
        - For RV(type=DSA), this is the publicly known group element whose discrete logarithm w.r.t. C(g) is the private key.
      type: int
      returned: When RV(type=DSA) or RV(type=ECC)
private_data:
  description:
    - Private key data. Depends on key type.
  returned: success and when O(return_private_key_data) is set to V(true)
  type: dict
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.privatekey_info import (
    PrivateKeyConsistencyError,
    PrivateKeyParseError,
    select_backend,
)


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "path": {"type": "path"},
            "content": {"type": "str", "no_log": True},
            "passphrase": {"type": "str", "no_log": True},
            "return_private_key_data": {"type": "bool", "default": False},
            "check_consistency": {"type": "bool", "default": False},
            "select_crypto_backend": {
                "type": "str",
                "default": "auto",
                "choices": ["auto", "cryptography"],
            },
        },
        required_one_of=(["path", "content"],),
        mutually_exclusive=(["path", "content"],),
        supports_check_mode=True,
    )

    result = {
        "can_load_key": False,
        "can_parse_key": False,
        "key_is_consistent": None,
    }

    if module.params["content"] is not None:
        data = module.params["content"].encode("utf-8")
    else:
        try:
            with open(module.params["path"], "rb") as f:
                data = f.read()
        except (IOError, OSError) as e:
            module.fail_json(
                msg=f"Error while reading private key file from disk: {e}",
                **result,  # type: ignore
            )

    result["can_load_key"] = True

    module_backend = select_backend(
        module=module,
        content=data,
        passphrase=module.params["passphrase"],
        return_private_key_data=module.params["return_private_key_data"],
        check_consistency=module.params["check_consistency"],
    )

    try:
        result.update(module_backend.get_info())
        module.exit_json(**result)
    except PrivateKeyParseError as exc:
        result.update(exc.result)
        module.fail_json(msg=exc.error_message, **result)  # type: ignore
    except PrivateKeyConsistencyError as exc:
        result.update(exc.result)
        module.fail_json(msg=exc.error_message, **result)  # type: ignore
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_webappvnetconnection
version_added: "1.8.0"
short_description: Manage web app virtual network connection
description:
    - Add, remove, or update the virtual network connection for a web app.
options:
    name:
        description:
            - Name of the web app.
        required: true
        type: str
    resource_group:
        description:
            - Resource group of the web app.
        required: true
        type: str
    state:
        description:
            - State of the virtual network connection. Use C(present) to create or update and C(absent) to delete.
        type: str
        default: present
        choices:
            - absent
            - present
    vnet_name:
        description:
            - Name of the virtual network. Required if adding or updating.
        type: str
    subnet:
        description:
            - Name of the virtual network's subnet. Required if adding or updating.
        type: str
    vnet_resource_group:
        description:
            - Name of the resource group for the virtual network. Defaults to main C(resource_group) value.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Configure web app with virtual network
  azure.azcollection.azure_rm_webappvnetconnection:
    name: "MyWebapp"
    resource_group: "MyResourceGroup"
    vnet_name: "MyVnetName"
    subnet: "MySubnetName"

- name: Configure web app with virtual network in different resource group
  azure.azcollection.azure_rm_webappvnetconnection:
    name: "MyWebapp"
    resource_group: "MyResourceGroup"
    vnet_name: "MyVnetName"
    subnet: "MySubnetName"
    vnet_resource_group: "MyOtherResourceGroup"

- name: Delete web app virtual network
  azure.azcollection.azure_rm_webappvnetconnection:
    name: "MyWebapp"
    resource_group: "MyResourceGroup"
    state: "absent"
'''

RETURN = '''
connection:
    description:
        - The web app's virtual network connection.
    returned: always
    type: complex
    contains:
        id:
            description:
                - ID of the web app virtual network connection.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/sites/myWebApp/virtualNetworkConnections/yyy-yyy_subnet
        name:
            description:
                - Name of the web app virtual network connection.
            returned: always
            type: str
            sample: yyy-yyy_subnet
        subnet_name:
            description:
               - Name of the subnet connected to the web app.
            returned: always
            type: str
            sample: mySubnet
        vnet_name:
            description:
               - Name of the virtual network connected to the web app.
            returned: always
            type: str
            sample: myVnet
        vnet_resource_group:
            description:
               - Name of the resource group the virtual network is in.
            returned: always
            type: str
            sample: myResourceGroup
        vnet_resource_id:
            description:
               - ID of the virtual network/subnet connected to the web app.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVnet/subnets/mySubnet
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.web.models import SwiftVirtualNetwork
except Exception:
    # This is handled in azure_rm_common
    pass


class AzureRMWebAppVnetConnection(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            vnet_name=dict(type='str'),
            subnet=dict(type='str'),
            vnet_resource_group=dict(type='str'),
        )

        self.results = dict(
            changed=False,
            connection=dict(),
        )

        self.state = None
        self.name = None
        self.resource_group = None
        self.vnet_name = None
        self.subnet = None
        self.vnet_resource_group = None

        super(AzureRMWebAppVnetConnection, self).__init__(self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=False)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        changed = False
        vnet = self.get_vnet_connection()
        if vnet:
            self.results['connection'] = self.set_results(vnet)

        if self.state == 'absent' and vnet:
            changed = True
            if not self.check_mode:
                self.log('Deleting vnet connection for webapp {0}'.format(self.name))
                self.delete_vnet_connection()
                self.results['connection'] = dict()
        elif self.state == 'present':
            self.vnet_resource_group = self.vnet_resource_group or self.resource_group

            if not vnet:
                self.log('Adding vnet connection for webapp {0}'.format(self.name))
                changed = True
            else:
                subnet_detail = self.get_subnet_detail(vnet.vnet_resource_id)
                if (subnet_detail['resource_group'] != self.vnet_resource_group
                        or subnet_detail['vnet_name'] != self.vnet_name
                        or subnet_detail['subnet_name'] != self.subnet):
                    self.log('Detected change in vnet connection for webapp {0}'.format(self.name))
                    changed = True

            if changed:
                if not self.check_mode:
                    self.log('Updating vnet connection for webapp {0}'.format(self.name))
                    subnet = self.get_subnet()
                    param = SwiftVirtualNetwork(subnet_resource_id=subnet.id)
                    self.create_or_update_vnet_connection(param)
                    vnet = self.get_vnet_connection()
                    self.results['connection'] = self.set_results(vnet)

        self.results['changed'] = changed
        return self.results

    def get_vnet_connection(self):
        connections = self.list_vnet_connections()
        for connection in connections:
            if connection.is_swift:
                return connection

        return None

    def list_vnet_connections(self):
        try:
            return self.web_client.web_apps.list_vnet_connections(resource_group_name=self.resource_group, name=self.name)
        except Exception as exc:
            self.fail("Error getting webapp vnet connections {0} (rg={1}) - {2}".format(self.name, self.resource_group, str(exc)))

    def delete_vnet_connection(self):
        try:
            return self.web_client.web_apps.delete_swift_virtual_network(resource_group_name=self.resource_group, name=self.name)
        except Exception as exc:
            self.fail("Error deleting webapp vnet connection {0} (rg={1}) - {3}".format(self.name, self.resource_group, str(exc)))

    def create_or_update_vnet_connection(self, vnet):
        try:
            return self.web_client.web_apps.create_or_update_swift_virtual_network_connection_with_check(
                resource_group_name=self.resource_group, name=self.name, connection_envelope=vnet)
        except Exception as exc:
            self.fail("Error creating/updating webapp vnet connection {0} (vnet={1}, rg={2}) - {3}".format(
                self.name, self.vnet_name, self.resource_group, str(exc)))

    def get_subnet(self):
        try:
            return self.network_client.subnets.get(resource_group_name=self.vnet_resource_group, virtual_network_name=self.vnet_name, subnet_name=self.subnet)
        except Exception as exc:
            self.fail("Error getting subnet {0} in vnet={1} (rg={2}) - {3}".format(self.subnet, self.vnet_name, self.vnet_resource_group, str(exc)))

    def set_results(self, vnet):
        vnet_dict = vnet.as_dict()

        output = dict()
        output['id'] = vnet_dict['id']
        output['name'] = vnet_dict['name']
        subnet_id = vnet_dict.get('subnet_resource_id', vnet_dict.get('vnet_resource_id'))
        output['vnet_resource_id'] = subnet_id
        subnet_detail = self.get_subnet_detail(subnet_id)
        output['vnet_resource_group'] = subnet_detail['resource_group']
        output['vnet_name'] = subnet_detail['vnet_name']
        output['subnet_name'] = subnet_detail['subnet_name']

        return output


def main():
    AzureRMWebAppVnetConnection()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Aparna Patil(@techcon65)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_diskencryptionset

version_added: "1.9.0"

short_description: Create, delete and update Disk encryption set

description:
    - Creates, deletes, and updates Disk encryption set.

options:
    resource_group:
        description:
            - The name of resource group.
        required: true
        type: str
    name:
        description:
            - The name of the disk encryption set.
        required: true
        type: str
    location:
        description:
            - Location for Disk encryption set. Defaults to location of resource group if not specified.
        type: str
    source_vault:
        description:
            - The name of source key vault containing encryption key.
        type: str
    key_url:
        description:
            - The url pointing to the encryption key to be used for disk encryption set.
        type: str
    identity:
        description:
            - Identity for the Object
        type: dict
        suboptions:
            type:
                description:
                    - Type of the managed identity
                choices:
                    - SystemAssigned
                    - UserAssigned
                    - SystemAssigned, UserAssigned
                default: SystemAssigned
                type: str
            user_assigned_identity:
                description:
                    - User Assigned Managed Identity associated to this resource
                required: false
                type: str
    state:
        description:
            - Assert the state of the disk encryption set. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Aparna Patil (@techcon65)
'''

EXAMPLES = '''
- name: create disk encryption set
  azure_rm_diskencryptionset:
    resource_group: myResourceGroup
    name: mydiskencryptionset
    source_vault: myvault
    key_url: https://myvault.vault.azure.net/keys/Key1/e65090b268ec4c3ba1a0f7a473005768
    state: present

- name: Update disk encryption set
  azure_rm_diskencryptionset:
    resource_group: myResourceGroup
    name: mydiskencryptionset
    source_vault: myvault
    key_url: https://myvault.vault.azure.net/keys/Key1/e65090b268ec4c3ba1a0f7a473005768
    state: present
    tags:
      key1: "value1"

- name: Delete disk encryption set
  azure_rm_diskencryptionset:
    resource_group: myResourceGroup
    name: mydiskencryptionset
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the Disk Encryption Set.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The disk encryption set ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/
                     Microsoft.Compute/diskEncryptionSets/mydiskencryptionset"
        name:
            description:
                - Disk encryption name.
            returned: always
            type: str
            sample: 'mydiskencryptionset'
        location:
            description:
                - The Azure Region where the resource lives.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: list
            sample: [{"key1": "value1"}]
        active_key:
            description:
                - Reference to Key vault and key used for disk encryption set.
            returned: always
            type: dict
            sample: {
                "key_url": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER",
                "source_vault": {
                    "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/
                           Microsoft.KeyVault/vaults/myvault"
                }
            }
        identity:
            description:
                - The managed identity for the disk encryption set.
            returned: always
            type: dict
            sample: {
                "principal_id": "d3abec0a-5818-4bbd-8300-8014198124ca",
                "tenant_id": "7268bab5-aabd-44f9-915f-6bf864e879c6",
                "type": "SystemAssigned"
            }
        provisioning_state:
            description:
                - The provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: "Microsoft.Compute/diskEncryptionSets"
'''

from ansible.module_utils.basic import _load_params
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import format_resource_id, normalize_location_name
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMDiskEncryptionSet(AzureRMModuleBaseExt):

    def __init__(self):

        _load_params()
        # define user inputs from playbook
        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            source_vault=dict(type='str'),
            key_url=dict(type='str', no_log=True),
            state=dict(choices=['present', 'absent'], default='present', type='str'),
            identity=dict(
                type="dict",
                options=self.managed_identity_single_required_spec
            )
        )

        required_if = [
            ('state', 'present', ['source_vault', 'key_url'])
        ]

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.source_vault = None
        self.key_url = None
        self.state = None
        self.tags = None
        self.identity = None
        self._managed_identity = None

        super(AzureRMDiskEncryptionSet, self).__init__(self.module_arg_spec,
                                                       required_if=required_if,
                                                       supports_check_mode=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {
                "identity": self.diskencryptionset_models.EncryptionSetIdentity,
                "user_assigned": self.diskencryptionset_models.UserAssignedIdentitiesValue,
            }
        return self._managed_identity

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False
        results = dict()
        disk_encryption_set = None

        # retrieve resource group to make sure it exists
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        self.location = normalize_location_name(self.location)

        if self.source_vault:
            source_vault = self.parse_resource_to_dict(self.source_vault)
            self.source_vault = format_resource_id(val=source_vault['name'],
                                                   subscription_id=source_vault['subscription_id'],
                                                   namespace='Microsoft.KeyVault',
                                                   types='vaults',
                                                   resource_group=source_vault['resource_group'])

        try:
            self.log('Fetching Disk encryption set {0}'.format(self.name))
            disk_encryption_set_old = self.diskencryptionset_client.disk_encryption_sets.get(self.resource_group,
                                                                                             self.name)
            # serialize object into a dictionary
            results = self.diskencryptionset_to_dict(disk_encryption_set_old)
            if self.state == 'present':
                changed = False
                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    changed = True
                self.tags = results['tags']
                if self.source_vault != results['active_key']['source_vault']['id']:
                    changed = True
                    results['active_key']['source_vault']['id'] = self.source_vault
                if self.key_url != results['active_key']['key_url']:
                    changed = True
                    results['active_key']['key_url'] = self.key_url
                if self.update_self_identity(old_identity=results["identity"]):
                    changed = True
            elif self.state == 'absent':
                changed = True

        except ResourceNotFoundError:
            if self.state == 'present':
                changed = True
                self.update_self_identity()
            else:
                changed = False

        self.results['changed'] = changed
        self.results['state'] = results

        if self.check_mode:
            return self.results

        if changed:
            if self.state == 'present':
                # create or update disk encryption set
                disk_encryption_set_new = \
                    self.diskencryptionset_models.DiskEncryptionSet(location=self.location,
                                                                    identity=self.identity)
                if self.source_vault:
                    source_vault = self.diskencryptionset_models.SourceVault(id=self.source_vault)
                    disk_encryption_set_new.active_key = \
                        self.diskencryptionset_models.KeyForDiskEncryptionSet(source_vault=source_vault,
                                                                              key_url=self.key_url)
                if self.tags:
                    disk_encryption_set_new.tags = self.tags
                self.results['state'] = self.create_or_update_diskencryptionset(disk_encryption_set_new)

            elif self.state == 'absent':
                # delete disk encryption set
                self.delete_diskencryptionset()
                self.results['state'] = 'Deleted'

        return self.results

    def update_self_identity(self, old_identity=None):
        safe_identity = self.identity or {'type': 'SystemAssigned'}
        update_identity, self.identity = self.update_single_managed_identity(
            curr_identity=old_identity,
            new_identity=safe_identity
        )
        return update_identity

    def create_or_update_diskencryptionset(self, disk_encryption_set):
        try:
            # create the disk encryption set
            response = \
                self.diskencryptionset_client.disk_encryption_sets.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                          disk_encryption_set_name=self.name,
                                                                                          disk_encryption_set=disk_encryption_set)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error creating or updating disk encryption set {0} - {1}".format(self.name, str(exc)))
        return self.diskencryptionset_to_dict(response)

    def delete_diskencryptionset(self):
        try:
            # delete the disk encryption set
            response = self.diskencryptionset_client.disk_encryption_sets.begin_delete(resource_group_name=self.resource_group,
                                                                                       disk_encryption_set_name=self.name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error deleting disk encryption set {0} - {1}".format(self.name, str(exc)))
        return response

    def diskencryptionset_to_dict(self, diskencryptionset):
        result = diskencryptionset.as_dict()
        result['tags'] = diskencryptionset.tags
        return result


def main():
    AzureRMDiskEncryptionSet()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_bgp_address_family_context_policy
short_description: Manage BGP address family context policy (bgp:CtxAfPol)
description:
- Manage BGP address family context policies for the Tenants on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  address_family_context_policy:
    description:
    - The name of the BGP address family context policy.
    type: str
    aliases: [ address_family_context_name, name ]
  host_route_leak:
    description:
    - The control state.
    - The option to enable/disable host route leak.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  ebgp_distance:
    description:
    - The administrative distance of eBGP routes.
    - The APIC defaults to C(20) when unset during creation.
    type: int
  ibgp_distance:
    description:
    - The administrative distance of iBGP routes.
    - The APIC defaults to C(200) when unset during creation.
    type: int
  local_distance:
    description:
    - The administrative distance of local routes.
    - The APIC defaults to C(220) when unset during creation.
    type: int
  ebgp_max_ecmp:
    description:
    - The eBGP max-path.
    - The APIC defaults to C(16) when unset during creation.
    type: int
  ibgp_max_ecmp:
    description:
    - The iBGP max-path.
    - The APIC defaults to C(16) when unset during creation.
    type: int
  local_max_ecmp:
    description:
    - The maximum number of equal-cost local paths for redist.
    - The APIC defaults to C(0) when unset during creation.
    - Can not be configured for APIC version 4.2(7s) and prior.
    type: int
  bgp_add_path_capability:
    description:
    - The neighbor system capability.
    - To delete this attribute, pass an empty string.
    - Can not be configured for APIC version 6.0(2h) and prior.
    type: str
    choices: [ receive, send, "" ]
  description:
    description:
    - Description for the BGP protocol profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(bgp:CtxAfPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Create a BGP address family context policy
  cisco.aci.aci_bgp_address_family_context_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    bgp_address_family_context_policy: my_bgp_address_family_context_policy
    host_route_leak: true
    ebgp_distance: 40
    ibgp_distance: 210
    local_distance: 215
    ebgp_max_ecmp: 32
    ibgp_max_ecmp: 32
    local_max_ecmp: 1
    bgp_add_path_capability: receive
    tenant: production
    state: present
  delegate_to: localhost

- name: Delete BGP address family context policy's child
  cisco.aci.aci_bgp_address_family_context_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    bgp_address_family_context_policy: my_bgp_address_family_context_policy
    bgp_add_path_capability: ""
    tenant: production
    state: absent
  delegate_to: localhost

- name: Delete a BGP address family context policy
  cisco.aci.aci_bgp_address_family_context_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    bgp_address_family_context_policy: my_bgp_address_family_context_policy
    tenant: production
    state: absent
  delegate_to: localhost

- name: Query all BGP address family context policies
  cisco.aci.aci_bgp_address_family_context_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific BGP address family context policy
  cisco.aci.aci_bgp_address_family_context_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    bgp_address_family_context_policy: my_bgp_address_family_context_policy
    tenant: production
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        address_family_context_policy=dict(type="str", aliases=["address_family_context_name", "name"]),  # Not required for querying all objects
        host_route_leak=dict(type="bool"),
        ebgp_distance=dict(type="int"),
        ibgp_distance=dict(type="int"),
        local_distance=dict(type="int"),
        ebgp_max_ecmp=dict(type="int"),
        ibgp_max_ecmp=dict(type="int"),
        local_max_ecmp=dict(type="int"),
        bgp_add_path_capability=dict(type="str", choices=["receive", "send", ""]),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["address_family_context_policy", "tenant"]],
            ["state", "present", ["address_family_context_policy", "tenant"]],
        ],
    )

    aci = ACIModule(module)

    address_family_context_policy = module.params.get("address_family_context_policy")
    host_route_leak = aci.boolean(module.params.get("host_route_leak"), "host-rt-leak", "")
    ebgp_distance = module.params.get("ebgp_distance")
    ibgp_distance = module.params.get("ibgp_distance")
    local_distance = module.params.get("local_distance")
    ebgp_max_ecmp = module.params.get("ebgp_max_ecmp")
    ibgp_max_ecmp = module.params.get("ibgp_max_ecmp")
    local_max_ecmp = module.params.get("local_max_ecmp")
    bgp_add_path_capability = module.params.get("bgp_add_path_capability")
    description = module.params.get("description")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    name_alias = module.params.get("name_alias")

    child_classes = []
    if bgp_add_path_capability is not None:
        child_classes.append("bgpCtxAddlPathPol")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="bgpCtxAfPol",
            aci_rn="bgpCtxAfP-{0}".format(address_family_context_policy),
            module_object=address_family_context_policy,
            target_filter={"name": address_family_context_policy},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if bgp_add_path_capability is not None:
            if bgp_add_path_capability == "" and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("bgpCtxAfPol", {}).get("children", {}):
                    if child.get("bgpCtxAddlPathPol"):
                        child_configs.append(dict(bgpCtxAddlPathPol=dict(attributes=dict(status="deleted"))))
            elif bgp_add_path_capability != "":
                child_configs.append(dict(bgpCtxAddlPathPol=dict(attributes=dict(capability=bgp_add_path_capability))))

        aci.payload(
            aci_class="bgpCtxAfPol",
            class_config=dict(
                name=address_family_context_policy,
                ctrl=host_route_leak,
                eDist=ebgp_distance,
                iDist=ibgp_distance,
                localDist=local_distance,
                maxEcmp=ebgp_max_ecmp,
                maxEcmpIbgp=ibgp_max_ecmp,
                maxLocalEcmp=local_max_ecmp,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="bgpCtxAfPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_confidential_compute
short_description: Confidential computing BIOS policy for Cisco Intersight
description:
  - Creates BIOS policies with hardware-level confidential computing features enabled.
  - Supports AMD SEV (Secure Encrypted Virtualization), AMD SEV-SNP, and Intel SGX
    (Software Guard Extensions) for secure AI enclaves.
  - Combines the correct BIOS knob settings needed to enable memory encryption and
    trusted execution environments on Cisco UCS servers.
  - Intended for AI workloads that require protection of proprietary model data in memory.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the BIOS policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the BIOS policy.
    type: str
    aliases: [descr]
  security_profile:
    description:
      - The confidential computing security profile to apply.
      - C(amd_sev) enables AMD Secure Encrypted Virtualization with memory encryption for VMs.
        Encrypts VM memory with per-VM keys so the hypervisor cannot read guest data.
        Requires AMD EPYC processors (Milan/Genoa or later).
      - C(amd_sev_snp) enables AMD SEV with Secure Nested Paging for stronger isolation.
        Adds integrity protection on top of SEV memory encryption, preventing hypervisor
        tampering with guest memory mappings. Requires AMD EPYC Genoa or later.
      - C(intel_sgx) enables Intel Software Guard Extensions for application-level enclaves.
        Creates hardware-encrypted memory enclaves that protect specific application data
        even from the OS and hypervisor. Requires Intel Xeon Scalable (Ice Lake or later).
      - C(intel_sgx_with_auto_reg) enables Intel SGX with automatic registration agent.
        Includes all intel_sgx settings plus enables the SGX auto-registration agent for
        simplified attestation service enrollment.
    type: str
    choices: [amd_sev, amd_sev_snp, intel_sgx, intel_sgx_with_auto_reg]
  sev_asid_count:
    description:
      - Number of AMD SEV Address Space Identifiers (ASIDs) to allocate.
      - Higher ASID counts allow more concurrent encrypted VMs but may reduce available ASIDs for SEV-SNP.
      - Only applicable for amd_sev and amd_sev_snp security profiles.
    type: str
    choices: [platform-default, '253 ASIDs', '509 ASIDs']
  sgx_epoch0:
    description:
      - Intel SGX Epoch 0 value. Changing this invalidates all existing SGX sealed data.
      - Only applicable for intel_sgx and intel_sgx_with_auto_reg profiles.
      - Use with caution in production environments.
    type: str
  sgx_epoch1:
    description:
      - Intel SGX Epoch 1 value. Changing this invalidates all existing SGX sealed data.
      - Only applicable for intel_sgx and intel_sgx_with_auto_reg profiles.
      - Use with caution in production environments.
    type: str
  enable_tme:
    description:
      - Override for Intel Total Memory Encryption (TME).
      - TME is a prerequisite for SGX and encrypts all system memory with a platform key.
      - Automatically enabled by intel_sgx profiles; set explicitly only to override.
    type: str
    choices: [platform-default, enabled, disabled]
  numa_optimized:
    description:
      - Override for NUMA optimization.
      - All confidential compute profiles enable NUMA optimization by default.
    type: str
    choices: [platform-default, enabled, disabled]
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Create AMD SEV policy for encrypted AI VMs
  cisco.intersight.intersight_confidential_compute:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: AI-Factory
    name: bios-amd-sev-ai
    description: AMD SEV for secure AI inference VMs
    security_profile: amd_sev
    sev_asid_count: "509 ASIDs"

- name: Create AMD SEV-SNP policy for maximum VM isolation
  cisco.intersight.intersight_confidential_compute:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-sev-snp-training
    description: SEV-SNP for secure model training
    security_profile: amd_sev_snp

- name: Create Intel SGX policy for enclave-based inference
  cisco.intersight.intersight_confidential_compute:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-sgx-inference
    description: SGX enclaves for confidential inference
    security_profile: intel_sgx_with_auto_reg

- name: Delete confidential compute policy
  cisco.intersight.intersight_confidential_compute:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-amd-sev-ai
    security_profile: amd_sev
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "bios-amd-sev-ai",
        "ObjectType": "bios.Policy",
        "Sev": "509 ASIDs",
        "CbsSevSnpSupport": "enabled"
    }
applied_settings:
  description: The BIOS settings applied by the security profile and any overrides.
  returned: when state is present
  type: dict
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


# Confidential computing security profile presets
SECURITY_PROFILES = {
    'amd_sev': {
        # AMD SEV - memory encryption for VMs
        'Sev': '509 ASIDs',
        # NUMA optimization for consistent memory locality
        'NumaOptimized': 'enabled',
        # IOMMU required for SEV
        'CbsDfCmnDramNps': 'Auto',
        # SMT enabled for VM density
        'CpuHwpm': 'HWPM Native Mode',
    },
    'amd_sev_snp': {
        # AMD SEV-SNP - memory encryption + integrity protection
        'Sev': '509 ASIDs',
        'CbsSevSnpSupport': 'enabled',
        'NumaOptimized': 'enabled',
        'CbsDfCmnDramNps': 'Auto',
        'CpuHwpm': 'HWPM Native Mode',
    },
    'intel_sgx': {
        # Intel SGX - application-level enclaves
        'EnableSgx': 'enabled',
        'EnableTme': 'enabled',
        'NumaOptimized': 'enabled',
        'CpuHwpm': 'HWPM Native Mode',
    },
    'intel_sgx_with_auto_reg': {
        # Intel SGX with auto-registration for attestation
        'EnableSgx': 'enabled',
        'EnableTme': 'enabled',
        'SgxAutoRegistrationAgent': 'enabled',
        'NumaOptimized': 'enabled',
        'CpuHwpm': 'HWPM Native Mode',
    },
}

# Map of override parameter names to API property names
OVERRIDE_MAP = {
    'enable_tme': 'EnableTme',
    'numa_optimized': 'NumaOptimized',
}


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        security_profile=dict(
            type='str',
            choices=['amd_sev', 'amd_sev_snp', 'intel_sgx', 'intel_sgx_with_auto_reg'],
        ),
        sev_asid_count=dict(type='str', choices=['platform-default', '253 ASIDs', '509 ASIDs']),
        sgx_epoch0=dict(type='str'),
        sgx_epoch1=dict(type='str'),
        enable_tme=dict(type='str', choices=['platform-default', 'enabled', 'disabled']),
        numa_optimized=dict(type='str', choices=['platform-default', 'enabled', 'disabled']),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/bios/Policies'

    intersight.api_body = {
        'Name': intersight.module.params['name'],
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
    }

    if module.params['state'] == 'present':
        if not module.params.get('security_profile'):
            module.fail_json(msg="security_profile is required when state is 'present'")

        intersight.set_tags_and_description()

        # Apply security profile preset
        profile_name = module.params['security_profile']
        profile_settings = SECURITY_PROFILES[profile_name].copy()

        # Apply SEV ASID count override for AMD profiles
        if module.params.get('sev_asid_count') is not None and profile_name in ('amd_sev', 'amd_sev_snp'):
            profile_settings['Sev'] = module.params['sev_asid_count']

        # Apply SGX epoch overrides for Intel profiles
        if profile_name in ('intel_sgx', 'intel_sgx_with_auto_reg'):
            if module.params.get('sgx_epoch0') is not None:
                profile_settings['SgxEpoch0'] = module.params['sgx_epoch0']
            if module.params.get('sgx_epoch1') is not None:
                profile_settings['SgxEpoch1'] = module.params['sgx_epoch1']

        # Apply generic overrides
        for param_name, api_property in OVERRIDE_MAP.items():
            if module.params.get(param_name) is not None:
                profile_settings[api_property] = module.params[param_name]

        intersight.api_body.update(profile_settings)
        intersight.result['applied_settings'] = profile_settings

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Samita Bhattacharjee (@samiib) <samitab@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_mcp_global_policy
short_description: Manage the MCP Global Policy in a Fabric Policy Template on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage the MisCabling Protocol (MCP) Global Policy in a Fabric Policy Template on Cisco Nexus Dashboard Orchestrator (NDO).
- There can only be a single MCP Global Policy in a Fabric Policy Template.
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Samita Bhattacharjee (@samiib)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Policy template.
    type: str
    aliases: [ fabric_template ]
    required: true
  name:
    description:
    - The name of the MCP Global Policy.
    type: str
    aliases: [ mcp_global_policy ]
  uuid:
    description:
    - The UUID of the MCP Global Policy.
    type: str
    aliases: [ mcp_global_policy_uuid ]
  description:
    description:
    - The description of the MCP Global Policy.
    - Providing an empty string will remove the O(description="") from the MCP Global Policy.
    type: str
  admin_state:
    description:
    - The administrative state of the MCP Global Policy.
    - Defaults to C(enabled) when unset during creation.
    type: str
    choices: [ enabled, disabled ]
  key:
    description:
    - The key of the MCP Global Policy.
    type: str
  per_vlan:
    description:
    - When enabled MCP will send packets on a per End Point Group (EPG) basis.
    - If disabled, the packets will only be sent on untagged EPGs which allows detecting loops in the native VLAN only.
    - Defaults to C(disabled) when unset during creation.
    type: str
    choices: [ enabled, disabled ]
    aliases: [ per_epg, mcp_pdu_per_vlan ]
  loop_detection_factor:
    description:
    - The amount of MCP packets that will be received before port disable loop protection action takes place.
    - Defaults to 3 when unset during creation.
    - The value must be between 1 and 255.
    type: int
    aliases: [ loop_factor, loop_detection_mult_factor ]
  port_disable:
    description:
    - Enable or disable port disabling when MCP packets are received.
    - Defaults to C(enabled) when unset during creation.
    type: str
    choices: [ enabled, disabled ]
    aliases: [ port_disable_protection ]
  initial_delay_time:
    description:
    - The MCP initial delay time in seconds.
    - Defaults to 180 when unset during creation.
    - The value must be between 0 and 1800.
    type: int
    aliases: [ initial_delay ]
  transmission_frequency_sec:
    description:
    - The MCP transmission frequency in seconds.
    - Defaults to 2 when unset during creation.
    - The value must be between 0 and 300.
    type: int
    aliases: [ tx_freq ]
  transmission_frequency_msec:
    description:
    - The MCP transmission frequency in milliseconds.
    - Defaults to 0 when unset during creation.
    - The value must be between 0 and 999.
    type: int
    aliases: [ tx_freq_ms ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Policy template.
- Attempts to create any additional MCP Global Policies will only update the existing
  object in the Fabric Policy template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create the MCP Global Policy object
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: example_mcp_global_policy
    state: present
  register: mcp_global_policy_new

- name: Create the MCP Global Policy object with all attributes
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: example_mcp_global_policy
    description: A Global MCP Policy
    key: cisco
    admin_state: enabled
    per_vlan: enabled
    loop_detection_factor: 3
    port_disable: enabled
    initial_delay_time: 180
    transmission_frequency_sec: 2
    transmission_frequency_msec: 10
    state: present
  register: mcp_global_policy_all

- name: Update the MCP Global Policy object with UUID
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: mcp_global_policy_update
    uuid: "{{ mcp_global_policy_all.current.uuid }}"
    state: present

- name: Query the MCP Global Policy object with name
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: example_mcp_global_policy
    state: query
  register: query_name

- name: Query the MCP Global Policy object with UUID
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    uuid: "{{ mcp_global_policy_all.current.uuid }}"
    state: query
  register: query_uuid

- name: Query the MCP Global Policy object in a Fabric Template
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    state: query
  register: query_all

- name: Delete the MCP Global Policy object with name
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: example_mcp_global_policy
    state: absent

- name: Delete the MCP Global Policy object with UUID
  cisco.mso.ndo_mcp_global_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    uuid: "{{ mcp_global_policy_all.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.constants import ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["fabric_template"]),
        name=dict(type="str", aliases=["mcp_global_policy"]),
        uuid=dict(type="str", aliases=["mcp_global_policy_uuid"]),
        description=dict(type="str"),
        admin_state=dict(type="str", choices=["enabled", "disabled"]),
        key=dict(type="str", no_log=False),
        per_vlan=dict(
            type="str",
            aliases=["per_epg", "mcp_pdu_per_vlan"],
            choices=["enabled", "disabled"],
        ),
        loop_detection_factor=dict(type="int", aliases=["loop_factor", "loop_detection_mult_factor"]),
        port_disable=dict(
            type="str",
            aliases=["port_disable_protection"],
            choices=["enabled", "disabled"],
        ),
        initial_delay_time=dict(type="int", aliases=["initial_delay"]),
        transmission_frequency_sec=dict(type="int", aliases=["tx_freq"]),
        transmission_frequency_msec=dict(type="int", aliases=["tx_freq_ms"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    # Enforcing that a user must specify a name or uuid when
    # adding, updating or removing even though there is only one policy per template.
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    name = module.params.get("name")
    uuid = module.params.get("uuid")
    state = module.params.get("state")
    per_vlan = module.params.get("per_vlan")
    port_disable = module.params.get("port_disable")

    mso_values = {
        "name": name,
        "description": module.params.get("description"),
        "adminState": module.params.get("admin_state"),
        "key": module.params.get("key"),
        "enablePduPerVlan": ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(per_vlan),
        "loopDetectMultFactor": module.params.get("loop_detection_factor"),
        "protectPortDisable": ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(port_disable),
        "initialDelayTime": module.params.get("initial_delay_time"),
        "txFreq": module.params.get("transmission_frequency_sec"),
        "txFreqMsec": module.params.get("transmission_frequency_msec"),
    }

    ops = []

    mso_template = MSOTemplate(mso, "fabric_policy", module.params.get("template"))
    mso_template.validate_template("fabricPolicy")

    object_description = "MCP Global Policy"
    object_base_path = "/fabricPolicyTemplate/template/mcpGlobalPolicy"

    existing_mcp_global_policy = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("mcpGlobalPolicy", {})

    if state in ["query", "absent"] and existing_mcp_global_policy == {}:
        mso.exit_json()

    elif state == "query" and not (name or uuid):
        mso.existing = [existing_mcp_global_policy]

    elif existing_mcp_global_policy and (name or uuid):
        mso.existing = mso.previous = copy.deepcopy(existing_mcp_global_policy)
        if uuid and mso.existing.get("uuid") != uuid:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

    if state == "present":
        if mso.existing:
            proposed_payload = copy.deepcopy(mso.existing)
            append_update_ops_data(ops, proposed_payload, object_base_path, mso_values)
            mso.sanitize(proposed_payload, collate=True)
        else:
            if uuid:
                mso.fail_json(msg="{0} cannot be created with a UUID".format(object_description))
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=object_base_path, value=mso.sent))

        mso.existing = mso.proposed

    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=object_base_path))

    if not module.check_mode and ops:
        response_object = mso.request(mso_template.template_path, method="PATCH", data=ops)
        mso.existing = response_object.get("fabricPolicyTemplate", {}).get("template", {}).get("mcpGlobalPolicy", {})
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2021, Phillipe Smith <phsmithcc@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard files documentation fragment
    DOCUMENTATION = r"""
options:
  url:
    type: str
    description:
      - Rundeck instance URL.
    required: true
  api_version:
    type: int
    description:
      - Rundeck API version to be used.
      - API version must be at least 14.
    default: 39
  api_token:
    type: str
    description:
      - Rundeck User API Token.
    required: true
"""



#!/usr/bin/python
#
# Copyright (c) 2024
# Nir Argaman <nargaman@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_vmsku_info

version_added: "2.4.0"

short_description: Get compute-related SKUs list

description:
    - Get details for compute-related resource SKUs.

options:
    location:
        description:
            - A region supported by current subscription.
        type: str
    resource_type:
        description:
            - Resource types e.g. "availabilitySets", "snapshots", "disks", etc.
        type: str
    size:
        description:
            - Size name, partial name is accepted.
        type: str
    zone:
        description:
            - Show skus supporting availability zones.
        type: bool
        default: False

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Nir Argaman (@nirarg)

'''

EXAMPLES = '''
- name: Gather Resource Group info
  azure.azcollection.azure_rm_resourcegroup_info:
    name: "{{ resource_group }}"
  register: rg_info

- name: List available VM SKUs
  azure.azcollection.azure_rm_vmsku_info:
    location: "{{ rg_info.resourcegroups.0.location }}"
    resource_type: "virtualMachines"
    size: "standard_B1"
    zone: true
  register: available_skus_result
'''

RETURN = '''
available_skus:
    description:
        - List of compute-related resource SKUs.
    returned: always
    type: complex
    contains:
        resource_type:
            description:
                - The type of resource the SKU applies to.
            returned: always
            type: str
            sample: "virtual_machine"
        name:
            description:
                - The name of SKU.
            returned: always
            type: str
            sample: "Standard_B1s"
        tier:
            description:
                - Specifies the tier of virtual machines in a scale set.
            returned: always
            type: str
            sample: "Standard"
        size:
            description:
            - The Size of the SKU.
            returned: always
            type: str
            sample: "B1s"
        family:
            description:
            - The Family of this particular SKU.
            returned: always
            type: str
            sample: "standardBSFamily"
        locations:
            description:
            - The set of locations that the SKU is available.
            returned: always
            type: list
            sample: ["eastus"]
        location_info:
            description:
                - A list of locations and availability zones in those locations where the SKU is available.
            returned: always
            type: complex
            contains:
                location:
                    description:
                        - Location of the SKU.
                    type: str
                    returned: always
                    sample: "eastus"
                zones:
                    description:
                        - List of availability zones where the SKU is supported.
                    type: list
                    returned: always
                    sample: ["1", "2", "3"]
                zone_details:
                    description:
                        - Details of capabilities available to a SKU in specific zones.
                    returned: always
                    type: complex
                    contains:
                        capabilities:
                            description:
                                - A list of capabilities that are available for the SKU in the specified list of zones.
                            type: complex
                            returned: always
                            contains:
                                name:
                                    description:
                                        - An invariant to describe the feature.
                                    type: str
                                    returned: always
                                    sample: "ultrassdavailable"
                                value:
                                    description:
                                        - An invariant if the feature is measured by quantity.
                                    type: str
                                    returned: always
                                    sample: "True"
        capabilities:
            description:
                - A name value pair to describe the capability.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - An invariant to describe the feature.
                    type: str
                    returned: always
                    sample: "ultrassdavailable"
                value:
                    description:
                        - An invariant if the feature is measured by quantity.
                    type: str
                    returned: always
                    sample: "True"
        restrictions:
            description:
                - The restrictions because of which SKU cannot be used. This is empty if there are no restrictions.
            returned: always
            type: complex
            contains:
                type:
                    description:
                        - The type of restrictions.
                    type: str
                    returned: always
                    sample: "location"
                restriction_values:
                    description:
                        - The value of restrictions. If the restriction type is set to location.
                          This would be different locations where the SKU is restricted.
                    type: list
                    returned: always
                    sample: ["eastus"]
                values:
                    description:
                        - The value of restrictions. If the restriction type is set to location.
                          This would be different locations where the SKU is restricted.
                        - This return value has been deprecated and will be removed in a release after
                          2027-05-01. Use C(restriction_values) instead.
                    type: list
                    returned: always
                    sample: ["eastus"]
                restriction_info:
                    description:
                        - The information about the restriction where the SKU cannot be used.
                    returned: always
                    type: complex
                    contains:
                        locations:
                            description:
                                - Locations where the SKU is restricted.
                            type: list
                            sample: ["location"]
                        zones:
                            description:
                                - List of availability zones where the SKU is restricted.
                            type: list
                            sample: ["1", "2"]
                reason_code:
                    description:
                        - The reason for restriction.
                    type: str
                    sample: "QuotaId"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.compute import ComputeManagementClient
    from azure.core.exceptions import HttpResponseError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMVmskuInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            location=dict(type='str'),
            resource_type=dict(type='str'),
            size=dict(type='str'),
            zone=dict(type='bool', default=False)
        )

        self.results = dict(
            available_skus=[],
            count=0
        )
        self.location = None
        self.resource_type = None
        self.size = None
        self.zone = False

        super(AzureRMVmskuInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                               supports_check_mode=True,
                                               supports_tags=False)

    def list_skus(self):
        try:
            compute_client = self.get_mgmt_svc_client(ComputeManagementClient,
                                                      base_url=self._cloud_environment.endpoints.resource_manager,
                                                      api_version='2021-07-01')
            skus_result = compute_client.resource_skus.list()
            available_skus = []
            for sku_info in skus_result:
                if self.location and not _match_location(self.location, sku_info.locations):
                    continue
                if not _is_sku_available(sku_info, self.zone):
                    continue
                if self.resource_type and not sku_info.resource_type.lower() == self.resource_type.lower():
                    continue
                if self.size and not (sku_info.resource_type == 'virtualMachines' and self.size.lower() in sku_info.name.lower()):
                    continue
                if self.zone and not (sku_info.location_info and sku_info.location_info[0].zones):
                    continue
                sku_dict = sku_info.as_dict()
                for restriction in sku_dict.get('restrictions', []):
                    if 'values' in restriction:
                        restriction['restriction_values'] = restriction['values']
                available_skus.append(sku_dict)
            return available_skus
        except HttpResponseError as e:
            # Handle exceptions
            raise e

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        available_skus = self.list_skus()
        self.results['available_skus'] = available_skus
        self.results['count'] = len(available_skus)
        self.module.deprecate(
            "The 'values' return key in 'restrictions' is deprecated. Use 'restriction_values' instead.",
            date="2027-05-01",
            collection_name="azure.azcollection",
        )
        return self.results


def _match_location(loc, locations):
    return next((x for x in locations if x.lower() == loc.lower()), None)


def _is_sku_available(sku_info, zone):
    """
    The SKU is unavailable in the following cases:
    1. regional restriction and the region is restricted
    2. parameter "zone" is input which indicates only showing skus with availability zones.
       Meanwhile, zonal restriction and all zones are restricted
    """
    is_available = True
    is_restrict_zone = False
    is_restrict_location = False
    if not sku_info.restrictions:
        return is_available
    for restriction in sku_info.restrictions:
        if restriction.reason_code == 'NotAvailableForSubscription':
            if restriction.type == 'Zone' and not (
                    set(sku_info.location_info[0].zones or []) - set(restriction.restriction_info.zones or [])):
                is_restrict_zone = True
            if restriction.type == 'Location' and (
                    sku_info.location_info[0].location in (restriction.restriction_info.locations or [])):
                is_restrict_location = True
            if is_restrict_location or (is_restrict_zone and zone):
                is_available = False
                break
    return is_available


def main():
    AzureRMVmskuInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

import operator
import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.fc_interfaces.fc_interfaces import (
    Fc_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.fc_interfaces import (
    Fc_interfacesTemplate,
)


__metaclass__ = type

"""
The nxos fc_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""


class Fc_interfacesFacts(object):
    """The nxos fc_interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Fc_interfacesArgs.argument_spec

    def get_interfaces_data(self, connection):
        return connection.get("show running-config interface all")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Fc_interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_interfaces_data(connection)

        # parse native config using the Fc_interfaces template
        fc_interfaces_parser = Fc_interfacesTemplate(lines=data.splitlines(), module=self._module)

        objs = list(fc_interfaces_parser.parse().values())

        # - populate only fc interfaces
        # - populate "analytics" value based on the presence or absense of "analytics_nvme" or "analytics_scsi" keys
        # - dummy key "m" and "p" is added for sorting, which is removed after sorting
        modified_objs = []
        # match only fc interface
        regex = re.compile(r"^fc\d+\S+")
        for parsed_data in objs:
            if not re.match(regex, parsed_data["name"]):
                continue
            m, p = parsed_data["name"].strip("fc").split("/")
            parsed_data["m"] = int(m)
            parsed_data["p"] = int(p)

            if "analytics_scsi" in parsed_data:
                parsed_data.pop("analytics_scsi")
                if "analytics_nvme" in parsed_data:
                    parsed_data.pop("analytics_nvme")
                    parsed_data["analytics"] = "fc-all"
                else:
                    parsed_data["analytics"] = "fc-scsi"
            else:
                if "analytics_nvme" in parsed_data:
                    parsed_data.pop("analytics_nvme")
                    parsed_data["analytics"] = "fc-nvme"
            modified_objs.append(parsed_data)

        sorted_dict = sorted(modified_objs, key=operator.itemgetter("m", "p"))
        objs = [
            {key: value for key, value in eachdict.items() if key not in ["m", "p"]}
            for eachdict in sorted_dict
        ]

        ansible_facts["ansible_network_resources"].pop("fc_interfaces", None)

        params = utils.remove_empties(
            fc_interfaces_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )

        facts["fc_interfaces"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts
